@@ -4,7 +4,7 @@ import inspect
 
 from sqlalchemy.orm.attributes import InstrumentedAttribute
 from sqlalchemy.orm import joinedload, aliased
-from sqlalchemy.sql.expression import desc, ColumnElement
+from sqlalchemy.sql.expression import desc
 from sqlalchemy import Boolean, Table, func, or_
 from sqlalchemy.exc import IntegrityError
 from sqlalchemy.sql.expression import cast
@@ -14,6 +14,7 @@ from flask import current_app, flash
 
 from flask_admin._compat import string_types, text_type
 from flask_admin.babel import gettext, ngettext, lazy_gettext
+from flask_admin.contrib.sqla.tools import is_relationship
 from flask_admin.model import BaseModelView
 from flask_admin.model.form import create_editable_list_form
 from flask_admin.actions import action
@@ -51,7 +52,7 @@ class ModelView(BaseModelView):
     """
 
     column_select_related_list = ObsoleteAttr('column_select_related',
-                                             'list_select_related',
+                                              'list_select_related',
                                               None)
     """
         List of parameters for SQLAlchemy `subqueryload`. Overrides `column_auto_select_related`
@@ -271,6 +272,16 @@ class ModelView(BaseModelView):
                 form_optional_types = (Boolean, Unicode)
     """
 
+    ignore_hidden = True
+    """
+       Ignore field that starts with "_"
+
+       Example::
+
+           class MyModelView(BaseModelView):
+               ignore_hidden = False
+    """
+
     def __init__(self, model, session,
                  name=None, category=None, endpoint=None, url=None, static_folder=None,
                  menu_class_name=None, menu_icon_type=None, menu_icon_value=None):
@@ -572,7 +583,7 @@ class ModelView(BaseModelView):
             raise Exception('Failed to find field for filter: %s' % name)
 
         # Figure out filters for related column
-        if hasattr(attr, 'property') and hasattr(attr.property, 'direction'):
+        if is_relationship(attr):
             filters = []
 
             for p in self._get_model_iterator(attr.property.mapper.class_):
@@ -603,9 +614,11 @@ class ModelView(BaseModelView):
 
             return filters
         else:
-            is_hybrid_property = isinstance(attr, ColumnElement)
+            is_hybrid_property = tools.is_hybrid_property(self.model, name)
             if is_hybrid_property:
                 column = attr
+                if isinstance(name, string_types):
+                    column.key = name.split('.')[-1]
             else:
                 columns = tools.get_columns_for_field(attr)
 
@@ -614,18 +627,35 @@ class ModelView(BaseModelView):
 
                 column = columns[0]
 
+            # If filter related to relation column (represented by
+            # relation_name.target_column) we collect here relation name
+            joined_column_name = None
+            if isinstance(name, string_types) and '.' in name:
+                joined_column_name = name.split('.')[0]
+
             # Join not needed for hybrid properties
             if (not is_hybrid_property and tools.need_join(self.model, column.table) and
                     name not in self.column_labels):
-                visible_name = '%s / %s' % (
-                    self.get_column_name(column.table.name),
-                    self.get_column_name(column.name)
-                )
+                if joined_column_name:
+                    visible_name = '%s / %s / %s' % (
+                        joined_column_name,
+                        self.get_column_name(column.table.name),
+                        self.get_column_name(column.name)
+                    )
+                else:
+                    visible_name = '%s / %s' % (
+                        self.get_column_name(column.table.name),
+                        self.get_column_name(column.name)
+                    )
             else:
                 if not isinstance(name, string_types):
                     visible_name = self.get_column_name(name.property.key)
                 else:
-                    visible_name = self.get_column_name(name)
+                    if self.column_labels and name in self.column_labels:
+                        visible_name = self.column_labels[name]
+                    else:
+                        visible_name = self.get_column_name(name)
+                        visible_name = visible_name.replace('.', ' / ')
 
             type_name = type(column.type).__name__
 
@@ -636,10 +666,19 @@ class ModelView(BaseModelView):
                 options=self.column_choices.get(name),
             )
 
+            key_name = column
+            # In case of filter related to relation column filter key
+            # must be named with relation name (to prevent following same
+            # target column to replace previous)
+            if joined_column_name:
+                key_name = "{0}.{1}".format(joined_column_name, column)
+                for f in flt:
+                    f.key_name = key_name
+
             if joins:
-                self._filter_joins[column] = joins
+                self._filter_joins[key_name] = joins
             elif not is_hybrid_property and tools.need_join(self.model, column.table):
-                self._filter_joins[column] = [column.table]
+                self._filter_joins[key_name] = [column.table]
 
             return flt
 
@@ -664,6 +703,7 @@ class ModelView(BaseModelView):
                                    only=self.form_columns,
                                    exclude=self.form_excluded_columns,
                                    field_args=self.form_args,
+                                   ignore_hidden=self.ignore_hidden,
                                    extra_fields=self.form_extra_fields)
 
         if self.inline_models:
@@ -788,9 +828,15 @@ class ModelView(BaseModelView):
             column = sort_field if alias is None else getattr(alias, sort_field.key)
 
             if sort_desc:
-                query = query.order_by(desc(column))
+                if isinstance(column, tuple):
+                    query = query.order_by(*map(desc, column))
+                else:
+                    query = query.order_by(desc(column))
             else:
-                query = query.order_by(column)
+                if isinstance(column, tuple):
+                    query = query.order_by(*column)
+                else:
+                    query = query.order_by(column)
 
         return query, joins
 
@@ -872,7 +918,9 @@ class ModelView(BaseModelView):
 
             # Figure out joins
             if isinstance(flt, sqla_filters.BaseSQLAFilter):
-                path = self._filter_joins.get(flt.column, [])
+                # If no key_name is specified, use filter column as filter key
+                filter_key = flt.key_name or flt.column
+                path = self._filter_joins.get(filter_key, [])
 
                 query, joins, alias = self._apply_path_joins(query, joins, path, inner_join=False)
 
@@ -892,7 +940,8 @@ class ModelView(BaseModelView):
                 spec = inspect.getargspec(flt.apply)
 
                 if len(spec.args) == 3:
-                    warnings.warn('Please update your custom filter %s to include additional `alias` parameter.' % repr(flt))
+                    warnings.warn('Please update your custom filter %s to '
+                                  'include additional `alias` parameter.' % repr(flt))
                 else:
                     raise
 
